;
; acat.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; acat [files]
;;
;;   Reads ascii files and outputs them as petscii. Converts plain text,
;;   expands tabs, converts cr/lf and removes control characters.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

WILDEXP		= 1

last		= $2
ichars		= $fb
bufp		= $fd
buffsize	= $2000

banner		= "asciicat 0.2 by soci/singular"

		.include "lib.asm"

		.section code
start2		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1
		lda #<obuffer
		sta k+1
		lda #>obuffer
		sta k+2
		lda #0
		sta bufp

loop		ldy #0
		lda (p1),y
		bne +
q		jsr out.flush
		jsr $ffcc
		jmp exit

+		ldx p1
		ldy p1+1
		pha
                jsr addstring
		pla
		inx
		bne +
		iny
+		jsr onefile
		ldx $91
		cpx #$7f
		beq q
		jmp loop

onefile		jsr $ffbd
		lda #1
		ldx mydrive
		ldy #0
		sty last
		jsr $ffba
		jsr $ffc0

		lda #<(ibuffer+buffsize-1)
		sta in.p+1
		lda #>(ibuffer+buffsize-1)
		sta in.p+2

		ldy #0
lp		ldx last
		cpx #"{tab}"
		bne +
		tya
		and #7
		cmp #7
		lda #32
		bcc k
		sta last
		gcs k

+		jsr in
		sta last
		asl
		cmp #32 << 1
		lda last
		bge nctrl

		cmp #"{tab}"
		beq lp
		cmp #"{cr}"
		beq k
		cmp #"{lf}"
		bne lp
		lda #"{cr}"
		cpx #"{cr}"
		bne k
                geq lp

nctrl		cmp #$41
		blt +
		cmp #$5b
		bge +
		ora #$80
		gmi k
+
		cmp #$61
		bge +
                cmp #$5f
                bne k
                lda #164
+		cmp #$7b
		bge k
		eor #$20

k		sta obuffer
		inc k+1
		beq out
k2		iny
		eor #13
		bne +
		tay
+		ldx $91
		cpx #$7f
		bne lp
		geq in.fin

out		.proc
		inc k+2
		ldx k+2
		cpx #>(obuffer+buffsize)
		bne k2
		jsr flush
		jmp k2

flush		pha
		tya
		pha

		ldx #3
		stx myout
		jsr $ffc9

		ldy #>obuffer
		sty bufp+1

		ldx k+1
		lda k+2
		sty k+2
		sec
		sbc #>obuffer
		tay
		lda #bufp
		jsr write

		pla
		tay
		pla
		rts
		.pend

in		.proc
		inc p+1
		bne +
		inc p+2
		lda p+2
		cmp #>(ibuffer+buffsize)
		bne +

		tya
		pha
		lda #>ibuffer
		sta bufp+1
		sta p+2

		ldx #1
		stx myin
		jsr $ffc6

		ldx #<buffsize
		ldy #>buffsize
		lda #bufp
		jsr read
		bcs ujj
		txa
		eor #255
		sta ichars
		tya
		eor #255
		sta ichars+1

		pla
		tay

+		inc ichars
		bne p
		inc ichars+1
		bne p
ujj		pla
		pla
fin		jsr $ffcc
		lda #1
		jmp $ffc3

p		lda ibuffer+buffsize-1
		rts
		.pend
		.send code

		.section bss
		.align 256
ibuffer		.fill buffsize
obuffer		.fill buffsize
		.send bss
