;
; cksum.asm
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;; cksum [files]
;;
;;   Calculates 32 bit CRC on files, and counts bytes. CRC algorithm
;;   is according to the POSIX standard. Can be used to check file
;;   integrity.
;;
;    This file is part of the IDE64 shell utilities.
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

WILDEXP		= 1

crctab0		= $c000
crctab1		= $c100
crctab2		= $c200
crctab3		= $c300

bufp            = $22
ichars		= $24
crc		= $fb
len		= $ac
ez		= $62
buffsize	= $4000

banner		= "cksum 0.2 by soci/singular"

		.include "lib.asm"

		.section code
crcgen          .macro
		eor crc+3
		tax
		lda crctab3,x
		eor crc+2
		sta crc+3
		lda crctab2,x
		eor crc+1
		sta crc+2
		lda crctab1,x
		eor crc
		sta crc+1
		lda crctab0,x
		sta crc
                .endm

start2		ldx #<arglist
		ldy #>arglist
		jsr parseargs
		lda #<arglist
		sta p1
		lda #>arglist
		sta p1+1

		ldx #0
-		lda #0
		sta crc+2
		sta crc+1
		sta crc+0
		txa
		ldy #8
-		asl crc
		rol crc+1
		rol crc+2
		rol
		bcc +
		eor #$04;$ed
		pha
		lda crc+2
		eor #$c1;$b8
		sta crc+2
		lda crc+1
		eor #$1d;$83
		sta crc+1
		lda crc+0
		eor #$b7;$20
		sta crc+0
		pla
+		dey
		bne -
		sta crctab3,x
		lda crc+2
		sta crctab2,x
		lda crc+1
		sta crctab1,x
		lda crc+0
		sta crctab0,x
		inx
		bne --

loop		ldy #0
		lda (p1),y
		bne +
q       	jmp exit

+		ldx p1
		ldy p1+1
		pha
                jsr addstring
		pla
		inx
		bne +
		iny
+		jsr onefile
		ldx $91
		cpx #$7f
		beq q
		jmp loop

onefile		jsr $ffbd
		lda #1
		ldx mydrive
		ldy #0
		jsr $ffba
		jsr $ffc0
		bcs err

		lda #0
		ldx #3
-		sta crc,x
		sta len,x
		dex
		bpl -

		jsr in
                bcc +
err             rts
+
		ldx #3
-		lda len,x
		sta $26,x
		dex
		bpl -

		lda len+3
-		ora len+2
		ora len+1
		ora len
		beq ki
		lda len
                jsr crcgen2
		lda len+1
		sta len
		lda len+2
		sta len+1
		lda len+3
		sta len+2
		lda #0
		sta len+3
		geq -
ki
		ldx #3
-		lda crc,x
		eor #255
		sta crc,x
		dex
		bpl -
		ldx #3
		jsr $ffc9
		bcs err

		ldx #crc
		jsr outnum
		ldx #$26
		jsr outnum
		ldy #0
-		cpy $b7
		beq +
		lda ($bb),y
		jsr $ffd2
		iny
		bne -
+		lda #13
		jsr $ffd2
		jmp $ffcc

crcgen2         #crcgen
                rts

outnum		.proc
		php
		sei
		sed
		lda #0
		ldy #4
-		sta ez,y
		dey
		bpl -

		jsr do
		jsr do
		jsr do
		jsr do

		plp

		ldx #128
		ldy #251
-		lda (ez-251) & $ffff,y
		jsr outhex
		iny
		bne -
		txa
		bpl +
		lda #$30
		jsr $ffd2
+		lda #32
		jmp $ffd2

do		lda $03,x
		sec
		rol
lp		pha
		ldy #4
-		lda ez,y
		adc ez,y
		sta ez,y
		dey
		bpl -
		pla
		asl
		bne lp
		dex
		rts
		.pend

outhex		.proc
		pha
		lsr
		lsr
		lsr
		lsr
		jsr o
		pla
o		and #$0f
		bne +
		inx
		bpl +
		rts
+		ora #$30
		tax
		jmp $ffd2
		.pend

in		.proc
		ldx #1
		stx myin
		jsr $ffc6

next            clc
		lda $90
                bne ujj
		lda #<buffer
		sta bufp
		lda #>buffer
		sta bufp+1
                sta p+2

		ldx #<buffsize
		ldy #>buffsize
		lda #bufp
		jsr read
		bcs ujj
		txa
                adc len
                sta len
                tya
                adc len+1
                sta len+1
                bcc +
                inc len+2
                bne +
                inc len+3
+
                inx
		stx ichars
		sty ichars+1

                ldy #0
-               dec ichars+1
                bmi last
p               lda buffer,y
                #crcgen
                iny
                bne p
		ldx $91
		cpx #$7f
		beq ujj
                inc bufp+1
                inc p+2
                gne -

last            dec ichars
                beq next
p2              lda (bufp),y
                jsr crcgen2
                iny
                jmp last

ujj             php
		jsr $ffcc
		lda #1
		jsr $ffc3
                plp
                rts
		.pend
		.send code

		.section bss
		.align 256
buffer		.fill buffsize
		.send bss
